"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const core_1 = require("@salesforce/core");
const azureDevOps_1 = require("../../helper/azureDevOps");
const utils_1 = require("../../helper/utils");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@slalom-salesforce/post-codereview-comments', 'azure.postcomments');
class PostComments extends sf_plugins_core_1.SfCommand {
    async run() {
        const { flags } = await this.parse(PostComments);
        const log = await core_1.Logger.child(this.ctor.name);
        // get current logger level for better console logs
        const logLevel = log.getLevel();
        // Azure DevOps related functionality is abstracted to AzureDevOps class
        const azure = new azureDevOps_1.default(flags.authtoken, flags.azureorgname);
        // maintains count of comments that already have been posted to the PR under consideration
        let countExistingComments = 0;
        // maintains count of net new comments that is posted through current execution
        let countNewCommentsPosted = 0;
        // get web url for PR
        const pullRequestUrl = await azure.getPrUrl(flags.azureprid, flags.azureprojectname);
        // reads static code analyzer output thats in JSON
        const scannerResults = (0, utils_1.getCodeReviewFromFile)(flags.reviewresultjsonfile);
        // transforms Scanner results into a cannonical model - CommentsByFile
        const CodeReviewCommentsByFiles = (0, utils_1.parseScannerOutput)(scannerResults, flags.foldernamefordiff);
        (0, utils_1.printMessage)(`CodeReviewCommentsByFiles -->
      ${JSON.stringify(CodeReviewCommentsByFiles, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
        // gets lines that have been updated in the current PR
        // this helps to filter comments to post only comments on lines that have been updated
        // since scanner reviews the entire file for issues
        const diffFilesLinesadded = await azure.getPrDiff(flags.azureprojectname, flags.azureprid, flags.gitfoldername);
        // fetch existing comments from PR into the cannonical model - CommentsByFile
        const existingCommentsByFiles = await azure.fetchPrComments(flags.azureprojectname, flags.azurereponame, flags.azureprid);
        (0, utils_1.printMessage)(`existingCommentsByFiles -->
      ${JSON.stringify(existingCommentsByFiles, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
        // gets count of existing comments
        if (existingCommentsByFiles && existingCommentsByFiles.length > 0) {
            countExistingComments = existingCommentsByFiles.length;
        }
        // compares code scanner results vs repository comments and returns array of just the net new files with file comments
        const newCommentsByFilesToPost = (0, utils_1.filterToNewCommentsToPost)(CodeReviewCommentsByFiles, existingCommentsByFiles);
        if (newCommentsByFilesToPost && newCommentsByFilesToPost.length > 0) {
            // returns filtered list of comments that are only applicable to the diff (lines that changed)
            const newCommentsByFilesToPostFilteredToDiff = (0, utils_1.filterCommentsToDiff)(diffFilesLinesadded, newCommentsByFilesToPost);
            (0, utils_1.printMessage)(`newCommentsByFilesToPostFilteredToDiff -->
        ${JSON.stringify(newCommentsByFilesToPostFilteredToDiff, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
            if (newCommentsByFilesToPostFilteredToDiff && newCommentsByFilesToPostFilteredToDiff.length > 0) {
                // creating thread
                void azure.postPrComment(flags.azureprojectname, flags.azurereponame, flags.azureprid, newCommentsByFilesToPostFilteredToDiff);
                // aggregate the count of comments across all files
                countNewCommentsPosted = newCommentsByFilesToPostFilteredToDiff
                    .map((eachFile) => eachFile.comments.length)
                    .reduce((previousValue, currentValue) => previousValue + currentValue, 0);
                (0, utils_1.printMessage)(`Posted ${countNewCommentsPosted} new comments to pr`, utils_1.MessageType.Success, logLevel);
            }
            else {
                (0, utils_1.printMessage)('No new code comments found to post', utils_1.MessageType.Warn, logLevel);
            }
        }
        else {
            (0, utils_1.printMessage)('No new code comments found to post', utils_1.MessageType.Warn, logLevel);
        }
        // Return an object to be displayed with --json
        return {
            existingCommentsCount: countExistingComments,
            newCommentsPosted: countNewCommentsPosted,
            link: pullRequestUrl,
        };
    }
}
exports.default = PostComments;
PostComments.summary = messages.getMessage('commandDescription');
PostComments.examples = messages.getMessages('examples');
// public static readonly args = [{ name: 'file' }];
// input flags for command
PostComments.flags = {
    authtoken: sf_plugins_core_1.Flags.string({
        char: 't',
        summary: messages.getMessage('authtokenDescription'),
        required: true,
    }),
    azureorgname: sf_plugins_core_1.Flags.string({
        char: 'o',
        summary: messages.getMessage('azureorgnameDescription'),
        required: true,
    }),
    azureprojectname: sf_plugins_core_1.Flags.string({
        char: 'p',
        summary: messages.getMessage('azureprojectnameDescription'),
        required: true,
    }),
    azurereponame: sf_plugins_core_1.Flags.string({
        char: 'r',
        summary: messages.getMessage('azurereponameDescription'),
        required: true,
    }),
    azureprid: sf_plugins_core_1.Flags.integer({
        char: 'i',
        summary: messages.getMessage('azurepridDescription'),
        required: true,
    }),
    foldernamefordiff: sf_plugins_core_1.Flags.directory({
        char: 'd',
        summary: messages.getMessage('foldernamefordiffDescription'),
        required: true,
    }),
    reviewresultjsonfile: sf_plugins_core_1.Flags.file({
        char: 'F',
        summary: messages.getMessage('reviewresultjsonfileDescription'),
        required: true,
    }),
    gitfoldername: sf_plugins_core_1.Flags.directory({
        char: 'g',
        summary: messages.getMessage('gitfoldernameDescription'),
        required: true,
    }),
};
//# sourceMappingURL=postcomments.js.map