"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const core_1 = require("@salesforce/core");
const bitbucketDevOps_1 = require("../../helper/bitbucketDevOps");
const utils_1 = require("../../helper/utils");
// Initialize Messages with the current plugin directory
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@slalom-salesforce/post-codereview-comments', 'bitbucket.postcomments');
class PostComments extends sf_plugins_core_1.SfCommand {
    async run() {
        const { flags } = await this.parse(PostComments);
        const log = await core_1.Logger.child(this.ctor.name);
        // get current logger level for better console logs
        const logLevel = log.getLevel();
        // printMessage(`flags -->${JSON.stringify(flags)}`, MessageType.Debug, logLevel);
        // Bitbucket related functionality is abstracted to BitbucketDevOps class
        const bitbucket = new bitbucketDevOps_1.default(flags.username, flags.authtoken, flags.bitbucketworkspace);
        // maintains count of comments that already have been posted to the PR under consideration
        let countExistingComments = 0;
        // maintains count of net new comments that is posted through current execution
        let countNewCommentsPosted = 0;
        (0, utils_1.printMessage)('fetching review comments from file', utils_1.MessageType.Debug, logLevel);
        const scannerResults = (0, utils_1.getCodeReviewFromFile)(flags.reviewresultjsonfile);
        // reads static code analyzer output thats in JSON
        (0, utils_1.printMessage)('parse scanner results', utils_1.MessageType.Debug, logLevel);
        // transforms Scanner results into a cannonical model - CommentsByFile
        const CodeReviewCommentsByFiles = (0, utils_1.parseScannerOutput)(scannerResults, flags.foldernamefordiff);
        (0, utils_1.printMessage)(`CodeReviewCommentsByFiles -->
        ${JSON.stringify(CodeReviewCommentsByFiles, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
        // gets lines that have been updated in the current PR
        // this helps to filter comments to post only comments on lines that have been updated
        // since scanner reviews the entire file for issues
        const diffFilesLinesadded = await bitbucket.getPrDiff(flags.bitbucketreposlug, flags.bitbucketprid, flags.gitfoldername);
        // fetch existing comments from PR into the cannonical model - CommentsByFile
        const existingCommentsByFiles = await bitbucket.fetchPrComments(flags.bitbucketreposlug, flags.bitbucketprid);
        (0, utils_1.printMessage)(`existingCommentsByFiles -->
        ${JSON.stringify(existingCommentsByFiles, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
        // gets count of existing comments
        if (existingCommentsByFiles && existingCommentsByFiles.length > 0) {
            countExistingComments = existingCommentsByFiles.length;
        }
        // compares code scanner results vs repository comments and returns array of just the net new files with file comments
        const newCommentsByFilesToPost = (0, utils_1.filterToNewCommentsToPost)(CodeReviewCommentsByFiles, existingCommentsByFiles);
        (0, utils_1.printMessage)(`diffFilesLinesadded -->
        ${JSON.stringify(diffFilesLinesadded, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
        if (newCommentsByFilesToPost && newCommentsByFilesToPost.length > 0) {
            const newCommentsByFilesToPostFilteredToDiff = (0, utils_1.filterCommentsToDiff)(diffFilesLinesadded, newCommentsByFilesToPost);
            (0, utils_1.printMessage)(`newCommentsByFilesToPostFilteredToDiff -->
          ${JSON.stringify(newCommentsByFilesToPostFilteredToDiff, undefined, 4)}`, utils_1.MessageType.Debug, logLevel);
            if (newCommentsByFilesToPostFilteredToDiff && newCommentsByFilesToPostFilteredToDiff.length > 0) {
                void bitbucket.postPrComment(flags.bitbucketreposlug, flags.bitbucketprid, newCommentsByFilesToPostFilteredToDiff);
                // aggregate the count of comments across all files
                countNewCommentsPosted = newCommentsByFilesToPostFilteredToDiff
                    .map((eachFile) => eachFile.comments.length)
                    .reduce((previousValue, currentValue) => previousValue + currentValue, 0);
                (0, utils_1.printMessage)(`Posted ${countNewCommentsPosted} new comments to ${newCommentsByFilesToPostFilteredToDiff.length} files on pr ${bitbucketDevOps_1.default.BITBUCKET_WEBAPP_URL}/${flags.bitbucketworkspace}/${flags.bitbucketreposlug}/pull-requests/${flags.bitbucketprid}`, utils_1.MessageType.Success, logLevel);
            }
            else {
                (0, utils_1.printMessage)('No new code comments found to post', utils_1.MessageType.Warn, logLevel);
            }
        }
        else {
            (0, utils_1.printMessage)('No new code comments found to post', utils_1.MessageType.Warn, logLevel);
        }
        return {
            existingCommentsCount: countExistingComments,
            newCommentsPosted: countNewCommentsPosted,
            link: `${bitbucketDevOps_1.default.BITBUCKET_WEBAPP_URL}/${flags.bitbucketworkspace}/${flags.bitbucketreposlug}/pull-requests/${flags.bitbucketprid}`,
        };
    }
}
exports.default = PostComments;
PostComments.PAGELENGTH = 50;
PostComments.summary = messages.getMessage('commandDescription');
PostComments.examples = messages.getMessages('examples');
PostComments.flags = {
    // flag with a value (-N, --username=VALUE)
    username: sf_plugins_core_1.Flags.string({
        char: 'N',
        summary: messages.getMessage('usernameDescription'),
        required: true,
    }),
    authtoken: sf_plugins_core_1.Flags.string({
        char: 't',
        summary: messages.getMessage('authtokenDescription'),
        required: true,
    }),
    bitbucketworkspace: sf_plugins_core_1.Flags.string({
        char: 'w',
        summary: messages.getMessage('bitbucketworkspaceDescription'),
        required: true,
    }),
    bitbucketreposlug: sf_plugins_core_1.Flags.string({
        char: 'r',
        summary: messages.getMessage('bitbucketreposlugDescription'),
        required: true,
    }),
    bitbucketprid: sf_plugins_core_1.Flags.integer({
        char: 'i',
        summary: messages.getMessage('bitbucketpridDescription'),
        required: true,
    }),
    foldernamefordiff: sf_plugins_core_1.Flags.directory({
        char: 'd',
        summary: messages.getMessage('foldernamefordiffDescription'),
        required: true,
    }),
    gitfoldername: sf_plugins_core_1.Flags.directory({
        char: 'g',
        summary: messages.getMessage('gitfoldernameDescription'),
        required: true,
    }),
    reviewresultjsonfile: sf_plugins_core_1.Flags.file({
        char: 'F',
        summary: messages.getMessage('reviewresultjsonfileDescription'),
        required: true,
    }),
};
//# sourceMappingURL=postcomments.js.map