"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeRefHeadFromBranchName = exports.parseDiff = exports.filterCommentsToDiff = exports.printMessage = exports.getCodeReviewFromFile = exports.filterToNewCommentsToPost = exports.parseScannerOutput = exports.getCommentBody = exports.MessageType = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const fs_1 = require("fs");
const core_1 = require("@salesforce/core");
const chalk = require("chalk");
// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-var-requires
const parser = require('git-pr-diff-parser');
// Initialize Messages with the current plugin directory
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@slalom-salesforce/post-codereview-comments', 'bitbucket.postcomments');
var MessageType;
(function (MessageType) {
    MessageType[MessageType["Info"] = 0] = "Info";
    MessageType[MessageType["Debug"] = 1] = "Debug";
    MessageType[MessageType["Success"] = 2] = "Success";
    MessageType[MessageType["Warn"] = 3] = "Warn";
    MessageType[MessageType["Error"] = 4] = "Error";
})(MessageType = exports.MessageType || (exports.MessageType = {}));
function getCommentBody(violation) {
    const commentBody = `Severity : ${violation.severity} \\n  \\n  ` +
        `Category : ${violation.category} \\n  \\n  ` +
        `Rule Name : ${violation.ruleName} \\n  \\n  ` +
        // eslint-disable-next-line no-control-regex
        `Issue : ${violation.message.replace(new RegExp('\n', 'g'), '')} \\n\\n  ` +
        `Reference : ${violation.url}`;
    return commentBody;
}
exports.getCommentBody = getCommentBody;
function parseScannerOutput(scannerResults, diffFolderName) {
    const parsedResult = [];
    scannerResults.forEach((scannerResult) => {
        const comments = [];
        scannerResult.violations.forEach((violation) => {
            const contentRaw = getCommentBody(violation);
            comments.push({
                lineNumber: parseInt(violation.line, 10),
                content: contentRaw,
                startLineNumber: parseInt(violation.line, 10),
                startColumnNumber: parseInt(violation.column, 10),
                endLineNumber: parseInt(violation.endLine, 10),
                endColumnNumber: parseInt(violation.endColumn, 10),
            });
        });
        let fileName = scannerResult.fileName;
        if (diffFolderName && fileName.startsWith(diffFolderName)) {
            fileName = fileName.split(diffFolderName)[1];
        }
        parsedResult.push({
            fileName,
            comments,
        });
    });
    return parsedResult;
}
exports.parseScannerOutput = parseScannerOutput;
function filterToNewCommentsToPost(CodeReviewCommentsByFilesToPost, existingCommentsByFiles) {
    let newCommentsByFilesToPost = [];
    if (CodeReviewCommentsByFilesToPost && CodeReviewCommentsByFilesToPost.length > 0) {
        if (existingCommentsByFiles && existingCommentsByFiles.length > 0) {
            CodeReviewCommentsByFilesToPost.forEach((codeReviewCommentsByFile) => {
                const matchedExistingCommentsByFile = existingCommentsByFiles.find((eachCommentsByFile) => eachCommentsByFile.fileName === codeReviewCommentsByFile.fileName);
                if (matchedExistingCommentsByFile) {
                    const tempNewCommentsByFileToPost = {
                        fileName: matchedExistingCommentsByFile.fileName,
                        comments: [],
                    };
                    codeReviewCommentsByFile.comments.forEach((reviewComment) => {
                        const matchedComment = matchedExistingCommentsByFile.comments.find((comment) => 
                        // eslint-disable-next-line no-control-regex
                        comment.content.replace(new RegExp('\n', 'g'), '\\n') === reviewComment.content &&
                            comment.lineNumber === reviewComment.lineNumber);
                        if (!matchedComment) {
                            tempNewCommentsByFileToPost.comments.push(reviewComment);
                        }
                    });
                    if (tempNewCommentsByFileToPost.comments.length > 0) {
                        newCommentsByFilesToPost.push(tempNewCommentsByFileToPost);
                    }
                }
                else {
                    newCommentsByFilesToPost.push(codeReviewCommentsByFile);
                }
            });
        }
        else {
            newCommentsByFilesToPost = CodeReviewCommentsByFilesToPost;
        }
    }
    return newCommentsByFilesToPost;
}
exports.filterToNewCommentsToPost = filterToNewCommentsToPost;
function getCodeReviewFromFile(filename) {
    let scannerResults;
    try {
        if (filename && (0, fs_1.existsSync)(filename)) {
            const rawdata = (0, fs_1.readFileSync)(filename, 'utf8');
            try {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                scannerResults = JSON.parse(rawdata);
            }
            catch (err) {
                throw new core_1.SfError(messages.getMessage('CodeReviewInputError'), 'CodeReviewInputError');
            }
        }
        else {
            throw new core_1.SfError(messages.getMessage('CodeReviewFileError'), 'CodeReviewInputError');
        }
    }
    catch (err) {
        throw new core_1.SfError(messages.getMessage('CodeReviewFileError'), 'CodeReviewInputError');
    }
    return scannerResults;
}
exports.getCodeReviewFromFile = getCodeReviewFromFile;
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function printMessage(message, messageType = MessageType.Info, loggerLevel) {
    if (messageType === MessageType.Info && loggerLevel <= core_1.LoggerLevel.INFO) {
        // eslint-disable-next-line no-console
        console.log(chalk.bgBlueBright.blue.bold(message));
    }
    else if (messageType === MessageType.Success && loggerLevel <= core_1.LoggerLevel.INFO) {
        // eslint-disable-next-line no-console
        console.log(chalk.bgGreenBright.green.bold(message));
    }
    else if (messageType === MessageType.Debug && loggerLevel <= core_1.LoggerLevel.DEBUG) {
        // eslint-disable-next-line no-console
        console.log(chalk.cyan.bold(message));
    }
    else if (messageType === MessageType.Warn && loggerLevel <= core_1.LoggerLevel.WARN) {
        // eslint-disable-next-line no-console
        console.log(chalk.bgYellowBright.yellow.bold(message));
    }
    else if (messageType === MessageType.Error && loggerLevel <= core_1.LoggerLevel.ERROR) {
        // eslint-disable-next-line no-console
        console.log(chalk.bgRedBright.red.bold(message));
    }
}
exports.printMessage = printMessage;
function filterCommentsToDiff(diffFilesLinesadded, newCommentsByFilesToPost) {
    const newCommentsByFilesToPostFilteredToDiff = [];
    newCommentsByFilesToPost.forEach((commentsByFile) => {
        let commentsByFileToPost;
        const commentsByFilesToPost = newCommentsByFilesToPostFilteredToDiff.filter((eachElement) => eachElement.fileName === commentsByFile.fileName);
        if (commentsByFilesToPost && commentsByFilesToPost.length > 0) {
            commentsByFileToPost = commentsByFilesToPost[0];
        }
        else {
            commentsByFileToPost = { fileName: commentsByFile.fileName, comments: [] };
        }
        const matchingDiff = diffFilesLinesadded.filter((diffFile) => diffFile.fileName === commentsByFile.fileName);
        if (matchingDiff && matchingDiff.length > 0) {
            commentsByFile.comments.forEach((comment) => {
                const index = matchingDiff[0].linesAdded.findIndex((lineNumber) => lineNumber === comment.lineNumber);
                if (index > -1) {
                    commentsByFileToPost.comments.push(comment);
                }
            });
        }
        if (commentsByFileToPost.comments.length > 0) {
            newCommentsByFilesToPostFilteredToDiff.push(commentsByFileToPost);
        }
    });
    return newCommentsByFilesToPostFilteredToDiff;
}
exports.filterCommentsToDiff = filterCommentsToDiff;
async function parseDiff(gitFolderPath, destinationBranchName, sourceBranchName) {
    const diffFilesLinesadded = [];
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call
    const parsedDiff = await parser(gitFolderPath, destinationBranchName, sourceBranchName);
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
    parsedDiff.forEach((eachFile) => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        let addedLine = eachFile.lines.filter((line) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-return
            if (line.type === 'add')
                return line;
        });
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        if (addedLine && addedLine.length > 0) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-call
            addedLine = addedLine.map((line) => line.ln);
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
        diffFilesLinesadded.push({ fileName: eachFile.to, linesAdded: addedLine });
    });
    return diffFilesLinesadded;
}
exports.parseDiff = parseDiff;
function removeRefHeadFromBranchName(branchRefName) {
    return branchRefName.startsWith('refs/heads/') ? branchRefName.substring(11) : branchRefName;
}
exports.removeRefHeadFromBranchName = removeRefHeadFromBranchName;
//# sourceMappingURL=utils.js.map